@@ -54,13 +54,19 @@ sub close {
 
 sub send_request_and_receive_response {
     my $self = shift;
-    
+
     if ($RMI::DEBUG) {
         print "$RMI::DEBUG_MSG_PREFIX N: $$ calling via $self: @_\n";
     }
     
     my $wantarray = wantarray;
-    $self->_send('query',[$wantarray, @_]) or die "failed to send! $!";
+
+    # Specs to get control of the serialization process are optional,  
+    # and may be at the beginning of the parameter list.  
+    # The remainder of the params are only analyzed on the caller's side.
+    my $opts = shift(@_) if ref($_[0]) eq 'HASH';
+
+    $self->_send('query',[$wantarray, @_],$opts) or die "failed to send! $!";
     
     for (1) {
         my ($message_type, $message_data) = $self->_receive();
@@ -110,8 +116,8 @@ sub receive_request_and_send_response {
 _mk_ro_accessors(qw/_sent_objects _received_objects _received_and_destroyed_ids _tied_objects_for_tied_refs/);
 
 sub _send {
-    my ($self, $message_type, $message_data) = @_;
-    my $s = $self->_serialize($message_type,$message_data);    
+    my ($self, $message_type, $message_data, $opts) = @_;
+    my $s = $self->_serialize($message_type,$message_data, $opts);    
     
     print "$RMI::DEBUG_MSG_PREFIX N: $$ sending: $s\n" if $RMI::DEBUG;
     return $self->{writer}->print($s,"\n");                
@@ -289,33 +295,99 @@ sub _respond_to_coderef {
 # serialize params when sending a query, or results when sending a response
 
 sub _serialize {
-    my ($self, $message_type, $message_data) = @_;    
-    
-    my $sent_objects = $self->{_sent_objects};
-    my $received_and_destroyed_ids = $self->{_received_and_destroyed_ids};
+    my ($self, $message_type, $message_data, $opts) = @_;    
+  
+    # TODO: we previously had no knowledge in the client of the type of call being made, 
+    # but to support overrides we now need it.  Refactor this conditional logic!!
+    if ($message_type eq 'query') {
+        my $call_type = $message_data->[1];
+        my $pkg;
+        my $sub;
+        if ($call_type eq 'call_object_method') {
+            $pkg = ref($message_data->[2]);
+            $pkg =~ s/RMI::Proxy:://;
+            $sub = $message_data->[3];
+        }
+        elsif ($call_type eq 'call_class_method') {
+            $pkg = $message_data->[2];
+            $sub = $message_data->[3];
+        }
+        elsif ($call_type eq 'call_function') {
+            ($pkg,$sub) = ($message_data->[2] =~ /^(.*)::([^\:]*)$/);
+        }
+        elsif (
+            $call_type eq 'call_eval'
+            or $call_type eq 'call_coderef'
+            or $call_type eq 'call_use'
+            or $call_type eq 'call_use_lib'
+        ) {
+            $pkg = '-' . $call_type;
+            if ($call_type eq 'call_use') {
+                $sub = $message_data->[2];
+                if (!$sub) {
+                    $sub = $message_data->[3];
+                    $sub =~ s/.pm$//;
+                    $sub =~ s/\//\::/g;
+                }
+                $sub .= '';
+            }
+            else {
+                $sub = $message_data->[2] . '';
+            }
+        }
+        else {
+            die "no handling for CALL TYPE $call_type?";
+        }
 
-    my @serialized = ([@$received_and_destroyed_ids]);
-    @$received_and_destroyed_ids = ();
-    
-    Carp::confess() unless ref($message_data);
+        unless ($pkg) {
+            die "Failed to resolve a pkg/sub pair for query @$message_data!";
+        }
+
+        my $default_opts = $RMI::ProxyObject::DEFAULT_OPTS{$pkg}{$sub};
+        print "$RMI::DEBUG_MSG_PREFIX N: $$ $message_type $call_type on $pkg $sub has default opts " . Data::Dumper::Dumper($default_opts) . "\n" if $RMI::DEBUG;
+        if ($default_opts) {
+            if ($opts) {
+                $opts = { %$default_opts, %$opts };
+                print "$RMI::DEBUG_MSG_PREFIX N: $$ $message_type $call_type on $pkg $sub merged with specified opts for combined set: " . Data::Dumper::Dumper($opts) . "\n" if $RMI::DEBUG;
+            }
+            else {
+                $opts = $default_opts;
+            }
+        }
+
+    }
+
+    my $sent_objects = $self->{_sent_objects};
+
+    # there is currently only one option to serialize: a global "copy" flag.
+    my $copy;
+    if ($opts) {
+        $copy = delete $opts->{copy};
+        if (%$opts) {
+            Carp::confess("Uknown options!  The only supported option is the 'copy' flag.");
+        }
+    }
+
+    my @serialized;
+    Carp::confess("expected message_type, message_data_arrayref, optional_opts_hashref as params") unless ref($message_data);
     for my $o (@$message_data) {
         if (my $type = ref($o)) {
-            if ($type eq "RMI::ProxyObject" or $RMI::proxied_classes{$type}) {
-                my $key = $RMI::Node::remote_id_for_object{$o};
+            # sending some sort of reference
+            if (my $key = $RMI::Node::remote_id_for_object{$o}) { 
+                # this is a proxy object on THIS side: the real object will be used on the remote side
                 print "$RMI::DEBUG_MSG_PREFIX N: $$ proxy $o references remote $key:\n" if $RMI::DEBUG;
                 push @serialized, 3, $key;
                 next;
             }
-            elsif ($type eq "RMI::ProxyReference") {
-                # This only happens from inside of AUTOLOAD in RMI::ProxyReference.
-                # There is some other reference in the system which has been tied, and this object is its
-                # surrogate.  We need to make sure that reference is deserialized on the other side.
-                my $key = $RMI::Node::remote_id_for_object{$o};
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ tied proxy special obj $o references remote $key:\n" if $RMI::DEBUG;
-                push @serialized, 3, $key;
-                next;
-            }            
+            elsif($copy) {
+                # a reference on this side which should be copied on the other side instead of proxied
+                # this never happens by default in the RMI modules, only when specially requested for performance
+                # or to get around known bugs in the C<->Perl interaction in some modules (DBI).
+                push @serialized, 4, $o;
+            }
             else {
+                # a reference originating on this side: send info so the remote side can create a proxy
+
                 # TODO: use something better than stringification since this can be overridden!!!
                 my $key = "$o";
                 
@@ -342,14 +414,26 @@ sub _serialize {
             }
         }
         else {
+            # sending a non-reference value
             push @serialized, 0, $o;
         }
     }
     print "$RMI::DEBUG_MSG_PREFIX N: $$ $message_type translated for serialization to @serialized\n" if $RMI::DEBUG;
 
     @$message_data = (); # essential to get the DESTROY handler to fire for proxies we're not holding on-to
+
+    # Do this after emptying the $message_data array, so the list will be expanded to include objects which
+    # were sent from the other side, and are only referenced in the data we're returning.
+    my $received_and_destroyed_ids = $self->{_received_and_destroyed_ids};
     print "$RMI::DEBUG_MSG_PREFIX N: $$ destroyed proxies: @$received_and_destroyed_ids\n" if $RMI::DEBUG;    
-    
+    unshift @serialized, [@$received_and_destroyed_ids];
+    @$received_and_destroyed_ids = ();
+
+    # TODO: the use of Data::Dumper here is pure laziness.  The @serialized list contains no references, 
+    # and could be turned into a string with something simpler than data dumper.  It could also be parsed with 
+    # something simpler than eval() on the other side.  The only thing to be careful of is that parsing 
+    # currently expects the records are divided by newlines (instead of sending a message length or other 
+    # terminator) and Dumper conveniently escapes newlines in any strings we pass.
     my $serialized_blob = Data::Dumper->new([[$message_type, @serialized]])->Terse(1)->Indent(0)->Useqq(1)->Dump;
     print "$RMI::DEBUG_MSG_PREFIX N: $$ $message_type serialized as $serialized_blob\n" if $RMI::DEBUG;
     if ($serialized_blob =~ s/\n/ /gms) {
@@ -363,8 +447,10 @@ sub _serialize {
 
 sub _deserialize {
     my ($self, $serialized_blob) = @_;
-    
+   
+    # see TODO above for switching from Dumper/eval to something simpler.
     my $serialized = eval "no strict; no warnings; $serialized_blob";
+
     if ($@) {
         die "Exception de-serializing message: $@";
     }        
@@ -432,7 +518,21 @@ sub _deserialize {
                         bless $o, $remote_class;
                     }
                     else {
-                        bless $o, 'RMI::ProxyObject';    
+                        # Put the object into a custom subclass of RMI::ProxyObject
+                        # this allows class-wide customization of how proxying should
+                        # occur.  It also makes Data::Dumper results more readable.
+                        my $target_class = 'RMI::Proxy::' . $remote_class;
+                        unless ($RMI::classes_with_proxied_objects{$remote_class}) {
+                            no strict 'refs';
+                            @{$target_class . '::ISA'} = ('RMI::ProxyObject');
+                            no strict;
+                            no warnings;
+                            local $SIG{__DIE__} = undef;
+                            local $SIG{__WARN__} = undef;
+                            eval "use $target_class";
+                            $RMI::classes_with_proxied_objects{$remote_class} = 1;
+                        }
+                        bless $o, $target_class;    
                     }
                 }
                 $received_objects->{$value} = $o;
@@ -459,6 +559,11 @@ sub _deserialize {
             push @message_data, $o;
             print "$RMI::DEBUG_MSG_PREFIX N: $$ - resolved local object for $value\n" if $RMI::DEBUG;
         }
+        elsif ($type == 4) {
+            # fully serialized blob
+            # this is never done by default, but is part of shortcut/optimization on a per-class basis
+            push @message_data, $value;
+        }
     }
     print "$RMI::DEBUG_MSG_PREFIX N: $$ remote side destroyed: @$received_and_destroyed_ids\n" if $RMI::DEBUG;
     my @done = grep { defined $_ } delete @$sent_objects{@$received_and_destroyed_ids};
@@ -577,7 +682,7 @@ RMI::Node - base class for RMI::Client and RMI::Server
 
 =head1 VERSION
 
-This document describes RMI::Node v0.09.
+This document describes RMI::Node v0.10.
 
 =head1 SYNOPSIS
     
@@ -657,24 +762,27 @@ Closes handles, and does any additional required bookeeping.
  
 =head2 send_request_and_recieve_response()
 
- @result = $n->send_request_and_recieve_response($call_type,$object,$method,$params,$opts)
+ @result = $n->send_request_and_recieve_response($call_type,@data);
 
- $fh = $n->send_request_and_receive_response('call_class_method', 'IO::File', 'new', ['/my/file'], {});
+ @result = $n->send_request_and_recieve_response($opts_hashref, $call_type, @data);
 
-This is the primary method used by nodes acting in a client-like capacity.
+ This is the method behind all of the call_* methods on RMI::Client objects.
+ It is also the method behind the proxied objects themselves (in AUTOLOAD).
 
- $call_type:    one of: call_object_method, call_class_method, or call_function, or one of several internal types
- $object:       the object or class on which the method is being called, may be undef for subroutine/function calls
- $method:       the method to call on $object (even if $object is a class name), or the fully-qualified sub name
- @params:       an optional array of values which should be passed to $method
+ The optional initial hashref allows special serialization control.  It is currently
+ only used to force serializing instead of proxying in some cases where this is
+ helpful and safe.
 
-Return values:
+ The call_type maps to the client request, and is one of:
+    call_function
+    call_class_method
+    call_object_method
+    call_eval
+    call_use
+    call_use_lib
 
- $result|@result: the return value will be either a scalar or list, depending on the value of $wantarray
-
-This method sends a method call request through the writer, and waits on a response from the reader.
-It will handle a response with the answer, exception messages, and also handle counter-requests
-from the server, which may occur b/c the server calls methods on objects passed as parameters.
+The interpretation of the @data parameters is dependent on the particular call_type, and
+is handled entirely on the remote side.  
 
 =head2 receive_request_and_send_response()
 
@@ -690,12 +798,12 @@ This method returns an anonymous subroutine which can be used in a "use lib $mys
 call, to cause subsequent "use" statements to go through this node to its partner.
  
  e.x.:
-    use lib RMI::Client::Tcp-new(host=>'myserver',port=>1234)->virtual_lib;
+    use lib RMI::Client::Tcp->new(host=>'myserver',port=>1234)->virtual_lib;
  
 If a client is constructed for other purposes in the application, the above
 can also be accomplished with: $client->use_lib_remote().  (See L<RMI::Client>)
 
-=head1 INTERNALS
+=head1 INTERNALS: MESSAGE TYPES
 
 The RMI internals are built around sending a "message", which has a type, and an
 array of data. The interpretation of the message data array is based on the message
@@ -735,7 +843,7 @@ The message data contains, in order:
 The return value from a succesful "query" which does not result in an
 exception being thrown on the remote side.
   
-The message data contains, the return value or vaues of that query.
+The message data contains the return value or values of that query.
   
 =head2 exception
 
@@ -749,7 +857,9 @@ Indicatees that the remote side has closed the connection.  This is actually
 constructed on the receiver end when it fails to read from the input stream.
   
 The message data is undefined in this case.
-  
+
+=head1 INTERNALS: WIRE PROTOCOL
+
 The _send() and _receive() methods are symmetrical.  These two methods are used
 by the public API to encapsulate message transmission and reception.  The _send()
 method takes a message_type and a message_data arrayref, and transmits them to
@@ -772,7 +882,7 @@ Each value is preceded by an integer which categorizes the value.
 
   0    a primitive, non-reference value
        
-       The value itself follows, and is passed by-copy.
+       The value itself follows, it is not a reference, and it is passed by-copy.
        
   1    an object reference originating on the sender's side
  
@@ -789,6 +899,18 @@ Each value is preceded by an integer which categorizes the value.
        The following value is the identifier the remote side sent previously.
        The remote side should substitue the original object when deserializing
 
+  4    a serialized object
+
+       This is the result of serializing the reference.  This happens only
+       when explicitly requested.  (DBI has some issues with proxies, for instance
+       and has customizations in RMI::Proxy::DBI::db to force serialization of
+       some connection attributes.)
+
+       See B<RMI::ProxyObject> for more details on forcing serialization.
+
+       Note that, because the current wire protocol is to use newline as a record 
+       separator, we use double-quoted strings to ensure all newlines are escaped.
+
 Note that all references are turned into primitives by the above process.
 
 =head2 stringification
